
# q-expansion of newform 5950.2.a.p, downloaded from the LMFDB on 17 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5950
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [1], [0], [-1], [-2], [5], [1], [1], [0], [1], [7], [-8], [12], [4], [1], [3], [5], [11], [-2], [-1], [1], [0], [12], [-13], [-5], [0], [16], [4], [-5], [3], [7], [-6], [12], [4], [6], [-12], [10], [-4], [14], [4], [20], [-10], [-12], [10], [-18], [5], [12], [-21], [23], [-14], [-21], [2], [4], [4], [28], [21], [-21], [-26], [-2], [-9], [-5], [19], [-20], [12], [-22], [-12], [-5], [5], [-15], [14], [0], [20], [-34], [-26], [4], [31], [-6], [-10], [-38], [-19], [-28], [-4], [8], [0], [-16], [-2], [-34], [40], [20], [9], [38], [-15], [-38], [23], [16], [6], [-40], [4], [-6], [-14], [9], [9], [-40], [3], [26], [32], [-48], [-6], [-40], [30], [32], [29], [27], [-42], [34], [38], [-12], [-21], [30], [31], [40], [-35], [48], [-15], [-40], [-24], [-27], [31], [-23], [-2], [35], [12], [5], [17], [-34], [41], [-10], [41], [-30], [6], [-32], [-17], [-22], [-24], [-18], [-51], [-6], [-13], [25], [-48], [-38], [-46], [-46], [42], [31], [32], [-50], [-2], [-32], [21], [27], [52], [24], [-27], [-6], [-6], [-29], [-60], [46], [14], [-63], [-22], [47], [36], [58], [-60], [54], [-51], [49], [10], [13], [-14], [-30], [28], [36], [-40], [8], [32], [-8], [-8], [-46], [-49], [-31], [-40], [23], [-30], [68], [-2], [57], [27], [42], [-9], [-14], [-55], [24], [46], [-50], [-38], [2], [7], [-53], [14], [-28], [8], [39], [56], [-17], [-35], [-20], [21], [-32], [-19], [-34], [10], [6], [4], [63], [-1], [21], [-66], [-31], [0], [54], [57], [26], [4], [-68], [-16], [12], [-4], [-33], [20], [40], [-28], [-3], [65], [8], [-52], [16], [-57], [53], [-32], [67], [-40], [21], [75], [-23], [36], [58], [2], [28], [18], [-14], [-20], [-73], [-64], [16], [-9], [-25], [26], [-17], [-44], [-10], [4], [-12], [5], [-72], [-6], [-69], [17], [-12], [-1], [30], [-62], [-44], [-32], [-69], [-22], [58], [-5], [37], [49], [0], [56], [50], [39], [82], [-51], [72], [-80], [44], [-1], [-79], [-45], [-46], [-14], [52], [-49], [-54], [27], [-32], [74], [-8], [-34], [51], [-25], [3], [-40], [30], [15], [-2], [35], [-66], [20], [-37], [-43], [2], [55], [52], [42], [62], [24], [-4], [74], [-8], [60], [0], [-22], [-12], [32], [9], [-6], [-23], [74], [-6], [26], [26], [-16], [-45], [36], [-4], [-74], [54], [14], [-80], [-90], [6], [-32], [42], [51], [-2], [0], [16], [-38], [66], [-28], [14], [93], [-60], [83], [-46], [44], [76], [8], [-38], [28], [-27], [100], [7], [-66], [-13], [78], [-26], [69], [-60], [-66], [-14], [10], [-13], [88], [-78], [90], [21], [12], [-74], [-78], [70], [66], [68], [24], [-34], [38], [-5], [77], [46], [-84], [17], [6], [12], [-98], [-87], [13], [-90], [8], [2], [-35], [12], [46], [-79], [-73], [6], [-78], [-90], [75], [41], [31], [-4], [2], [77], [-56]]
